--vim: set et ts=4 :
--[[
The purpose of this addon is to enable the player to use specific mounts and
companions for areas in the game.
Whereas AltzUI has its 'autopet' option which selects a pet from the global
list or from the favorite list, this addon can do it for each configured area.

The pets get summoned automatically.
For mounts, the player should set up three macro buttons for the commands
/aaf mount ground, /aaf mount flying, /aaf mount swimming, respectively.

For configuration, you can use the /aaf command, which presents a dialog
with buttons for remembering the pets and the mounts.
--]]

-- Stores the user's data. Each character has its own copy of this database.
AutoAreaFavoritesDB = {
    -- Keyed by zone_id (number), stores a map with keys
    -- 'pet', 'flying', 'ground', 'swimming'
    zone_to_set = {}
}

--[[
Wraps GetCurrentMapAreaID because the function needs a call to
SetMapToCurrentZone for proper return values.
--]]
local function get_real_zone_id()
    SetMapToCurrentZone()
    local zone_id, is_continent = GetCurrentMapAreaID()
    if is_continent then
        zone_id = nil
    end
    return zone_id
end

--[[
Called by slash command /AAF_rememberpet
Associates the currently summoned pet with the current zone.
When there is no summoned pet, the command removes the association.
To deassociate a pet then:
1) dismiss the pet
2) remember the pet
--]]
function AutoAreaFavorites_remember_pet()
    local zone_id = get_real_zone_id()
    if zone_id then
        local current_pet = C_PetJournal.GetSummonedPetGUID()
        if current_pet then
            if not AutoAreaFavoritesDB.zone_to_set[zone_id] then
                AutoAreaFavoritesDB.zone_to_set[zone_id] = {}
            end
        end
        AutoAreaFavoritesDB.zone_to_set[zone_id].pet = current_pet
    end
end

--[[
Called on player events to summon a pet which is saved for a zone.
--]]
function AutoAreaFavorites_summon_pet()
    local zone_id = get_real_zone_id()
    if zone_id then
        local zone_favorites = AutoAreaFavoritesDB.zone_to_set[zone_id]
        if zone_favorites and zone_favorites.pet then
            local pet_id = zone_favorites.pet
            local summoned_pet = C_PetJournal.GetSummonedPetGUID()
            if summoned_pet then
                if summoned_pet == pet_id then
                    -- Tricky. The zone's pet is currently summoned.
                    -- This seems to happen when we get called several times
                    -- in a row. For now, just do nothing.
                    return
                else
                    -- Dismiss by summon...yeah
                    C_PetJournal.SummonPetByGUID(summoned_pet)
                end
            end
            C_PetJournal.SummonPetByGUID(pet_id)
        end
    end
end

--[[
Called by slash command /AAF_remembermount <kind>
Associates the current mount with the current zone and usage type.

<kind> can be one of ground, flying or swimming. The parameter is checked
       before it is used here.

When issued unmounted, the command disassociates the mount kind from this zone.
To deassociate a mount then:
1) dismount
2) remember the mount
--]]
function AutoAreaFavorites_remember_mount(mkind)
    local zone_id = get_real_zone_id()
    if zone_id then
        -- Find the currently active mount
        local mount_id = nil
        if IsMounted() then
            local mount_info = nil
            -- TODO:
            -- The duo C_MountJournal.GetMountIDs C_MountJournal.GetMountInfoByID
            -- does not work properly.
            for dmi = 1, C_MountJournal.GetNumDisplayedMounts() do
                mount_info = {C_MountJournal.GetDisplayedMountInfo(dmi)}
                local active = mount_info[4]
                if active then
                    mount_id = mount_info[12]
                    break
                end
            end
            if mount_info then
                if not AutoAreaFavoritesDB.zone_to_set[zone_id] then
                    AutoAreaFavoritesDB.zone_to_set[zone_id] = {}
                end
            end
        end
        AutoAreaFavoritesDB.zone_to_set[zone_id][mkind] = mount_id
    end
end

--[[
Called by slash command /AAF mount <kind>
Summons the mount which was saved for this mode in this current zone.
Unknown or missing kind results in 'ground'.

<kind> can be one of ground, flying or swimming
--]]
function AutoAreaFavorites_summon_mount(kind)
    local zone_id = get_real_zone_id()
    if zone_id then
        local zone_favorites = AutoAreaFavoritesDB.zone_to_set[zone_id]
        local mount_id = 0
        if zone_favorites and zone_favorites[kind] then
            mount_id = zone_favorites[kind]
        end
        C_MountJournal.SummonByID(mount_id)
    end
end

--[[
Parses the slash command and invokes the functions.
--]]
local function handle_slash_command(text, _)
    local tokens = { strsplit(' ', text) }
    if #tokens > 0 then
        local cmd = string.lower(tokens[1])
        if cmd == 'rememberpet' or cmd == 'savepet' or cmd == 'forgetpet' then
            remember_pet()
        elseif cmd == 'summonpet' then
            summon_pet()
        elseif cmd == 'remembermount' or cmd == 'savemount' or cmd == 'forgetmount' or cmd == 'summonmount' or cmd == 'mount' then
            -- default to ground mount
            local mount_kind = 'ground'
            local user_mount_kind = tokens[2]
            if user_mount_kind then
                user_mount_kind = string.lower(user_mount_kind)
                local usable_kinds = {'ground', 'flying', 'swimming'}
                for _, v in pairs(usable_kinds) do
                    if user_mount_kind == v then
                        mount_kind = user_mount_kind
                        break
                    end
                end
            end
            if cmd == 'summonmount' or cmd == 'mount' then
                summon_mount(mount_kind)
            else
                remember_mount(mount_kind)
            end
        else
            ShowUIPanel(AutoAreaFavoritesMenu)
        end
    end
end

--[[
Pet autosummon events.
These were picked straight from AltZ UI.
--]]
local pet_autosummon_events = {
    ['PLAYER_ENTERING_WORLD'] = true,
    ['PLAYER_CONTROL_GAINED'] = true,
    ['UNIT_EXITED_VEHICLE'] = true,
    ['PLAYER_ALIVE'] = true
}

--[[
This is the main initialization function.
It responds to the loaded event, and then it sets up further events and
commands.
--]]
function AutoAreaFavorites_OnLoad(self)
    self:RegisterForDrag('LeftButton');
    self:SetUserPlaced(1);
    self:RegisterEvent('ADDON_LOADED');
    -- Pet autosummon events
    for event, _ in pairs(pet_autosummon_events) do
        self:RegisterEvent(event)
    end
    -- Our slash commands
    _G.SLASH_AUTOAREAFAVORITES1 = '/AutoAreaFavorites'
    _G.SLASH_AUTOAREAFAVORITES2 = '/aaf'
    _G.SLASH_AUTOAREAFAVORITES3 = '/AAF'
    _G.SlashCmdList['AUTOAREAFAVORITES'] = handle_slash_command
end

function AutoAreaFavorites_OnEvent(frame, event, ...)
    local result = false
    if pet_autosummon_events[event] then
        -- AltzUI uses a delay for this, so can we. But why is this necessary?
        C_Timer.After(5, AutoAreaFavorites_summon_pet)
        result = true
    end
    return result
end
